import sys, os
from sklearn import datasets
import numpy as np

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from chembee_config.benchmark.svc import SVClassifier
from chembee_config.benchmark.svc_poly import SVCPolyClassifier
from chembee_config.benchmark.spectral_clustering import SpectralClusteringClassifier
from chembee_config.benchmark.random_forest import RandomForestClassifier
from chembee_config.benchmark.naive_bayes import NaiveBayesClassifier
from chembee_config.benchmark.logistic_regression import LogisticRegressionClassifier
from chembee_config.benchmark.linear_regression import LinearRegressionClassifier
from chembee_config.benchmark.kmeans import KMeansClassifier
from chembee_config.benchmark.knn import KNNClassifier
from chembee_config.benchmark.mlp_classifier import NeuralNetworkClassifier
from chembee_config.benchmark.restricted_bm import RBMClassifier

from chembee_plotting.graphics import plotting_map_comparison
from file_utils import save_json_to_file
import logging
from dotenv import load_dotenv


load_dotenv()

logging.basicConfig(
    format="%(levelname)s:%(asctime)s %(message)s",
    datefmt="%m/%d/%Y %I:%M:%S %p",
    level=logging.INFO,
    file_name="chembee.log",
)


def benchmark_standard(
    X,
    y,
    feature_names=["Feature 1", "Feature 2"],
    out_name="benchmark",
    prefix="plots/benchmarks",
):
    algorithms = [
        SVClassifier,
        SVCPolyClassifier,
        SpectralClusteringClassifier,
        RandomForestClassifier,
        NaiveBayesClassifier,
        LogisticRegressionClassifier,
        LinearRegressionClassifier,
        KMeansClassifier,
        KNNClassifier,
        NeuralNetworkClassifier,
        RBMClassifier,
    ]
    metrics = {}
    for alg in algorithms:
        metric = benchmark_algorithm(
            alg,
            X,
            y,
            plot_function=plotting_map_comparison[len(alg.algorithms)],
            file_name="benchmark_" + alg.name,
            prefix=prefix,
            feature_names=feature_names,
            response_method=alg._response_method,
        )
    save_json_to_file(metrics, file_name=out_name, prefix=prefix)
    return metrics


def benchmark_algorithm(
    algorithm,
    X: np.ndarray,
    y: np.ndarray,
    plot_function: object(),
    file_name: str = "benchmark",
    prefix: str = "benchmarks/",
    feature_names: list = ["Feature 1", "Feature 2"],
    response_method: str = "predict",
) -> dict:
    name = algorithm.name
    algorithms = algorithm.algorithms
    file_name = "benchmark_" + name
    metrics_class = []
    models = []
    for i in range(len(algorithm.titles)):
        model = algorithms[i].fit(X, y)
        models.append(model)
        if response_method == "predict":
            y_pred = model.predict(X)
        elif response_method == "fit_predict":
            y_pred = model.fit_predict(X, y)
        if response_method == "transform":
            y_pred = model.transform(X)
        logging.info("Fitted model: ", algorithm.titles[i])

    plot_function(
        models=models,
        titles=algorithm.titles,
        X=X,
        y=y,
        file_name=file_name,
        prefix=prefix,
        feature_names=feature_names,
        response_method=response_method,
    )

    metrics = {k: v for k, v in zip(algorithm.titles, metrics_class)}
    return metrics



import sys, os
from sklearn import datasets

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from utils import prepare_data_decicion_lib
from chembee_actions.benchmark_algorithms import benchmark_standard


def test_benchmarking():

    iris = datasets.load_iris()
    X, y = prepare_data_decicion_lib(iris)
    benchmark_standard(
        X,
        y,
        feature_names=iris.feature_names[:2],
    )



import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from sklearn import svm


from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class SVCPolyClassifier(BenchmarkAlgorithm):

    name = "svc_polys"
    C = 1.0
    algorithms = (
        svm.SVC(kernel="poly", degree=1, gamma="auto", C=C),
        svm.SVC(kernel="poly", degree=3, gamma="auto", C=C),
        svm.SVC(kernel="poly", degree=6, gamma="auto", C=C),
    )

    titles = (
        "Polynomial (degree 1) kernel",
        "Polynomial (degree 3) kernel",
        "Polynomial (degree 6) kernel",
    )



import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from sklearn.cluster import KMeans


from chembee_config.benchmark.BenchmarkAlgorithm import BenchmarkAlgorithm


class KMeansClassifier(BenchmarkAlgorithm):

    name = "kmeans"

    algorithms = (
        KMeans(
            n_clusters=3,
            init="k-means++",
            n_init=10,
            max_iter=300,
            tol=0.0001,
            verbose=0,
            random_state=None,
            copy_x=True,
            algorithm="lloyd",
        ),
        KMeans(
            n_clusters=3,
            init="random",
            n_init=10,
            max_iter=300,
            tol=0.0001,
            verbose=0,
            random_state=None,
            copy_x=True,
            algorithm="lloyd",
        ),
        KMeans(
            n_clusters=3,
            init="k-means++",
            n_init=10,
            max_iter=300,
            tol=0.0001,
            verbose=0,
            random_state=None,
            copy_x=True,
            algorithm="lloyd",
        ),
        KMeans(
            n_clusters=3,
            init="k-means++",
            n_init=10,
            max_iter=300,
            tol=0.0001,
            verbose=0,
            random_state=None,
            copy_x=True,
            algorithm="elkan",
        ),
    )

    titles = (
        "k-means++ initialization",
        "random initialization",
        "Lloyd method",
        "Elkan method",
    )



# want to do with ROC check: https://scikit-learn.org/stable/auto_examples/model_selection/plot_roc_crossval.html#sphx-glr-auto-examples-model-selection-plot-roc-crossval-py
# making own scoring: https://scikit-learn.org/stable/modules/cross_validation.html
# Want to do Stratified k-fold
from sklearn.model_selection import train_test_split
from sklearn.model_selection import GridSearchCV
from sklearn.metrics import classification_report

# algorithm

# cross validation parameters
import logging


logging.basicConfig(
    format="%(levelname)s:%(asctime)s %(message)s",
    datefmt="%m/%d/%Y %I:%M:%S %p",
    level=logging.INFO,
    filename="chembee_actions.log",
)


def cross_validation_grid_search(scores: list, clf, X_train, X_test, y_train, y_test):

    # is property of algorithm

    for score in scores:

        clf = GridSearchCV(
            clf, clf.hyperparameters, scoring="%s_macro" % score, n_jobs=-1
        )
        clf.fit(X_train, y_train)
        y_true, y_pred = y_test, clf.predict(X_test)

        return clf, clf.cv_results_, clf.best_params_, clf.best_index_



import sys, os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from chembee_datasets.BreastCancer import BreastCancerDataset
from chembee_actions.calibration import screen_calibration


def test_calibration():

    DataSet = BreastCancerDataset(split_ratio=0.8)
    screen_calibration(
        DataSet.X_train,
        DataSet.X_test,
        DataSet.y_train,
        DataSet.y_test,
        file_name="breast_cancer_calibration",
        prefix="plots/calibration",
    )



import sys, os

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from chembee_actions.cross_validation import cross_validation_grid_search
from chembee_datasets.BioDegDataSet import BioDegDataSet
from chembee_actions import cross_validation
from chembee_preparation.processing import calculate_lipinski_desc
from chembee_config.calibration.random_forest import RandomForestClassifier


def test_cross_validation():

    """
    The benchmark tests cross validation on the random forest classifier based on
    parameters defined in the configuration file of the algorithm
    """

    file_name = "test"
    prefix = "tests/plots"
    target = "ReadyBiodegradability"
    split_ratio = 0.7
    scores = [
        "precision",
        "recall",
    ]  # need to specify the loss functions for the crosss validation
    DataSet = BioDegDataSet(
        split_ratio=split_ratio, data_set_path="tests/data/Biodeg.sdf", target=target
    )
    data = calculate_lipinski_desc(DataSet.data, DataSet.mols)
    data = DataSet.clean_data(DataSet.data)
    (
        DataSet.X_train,
        DataSet.X_test,
        DataSet.y_train,
        DataSet.y_test,
    ) = DataSet.make_train_test_split(
        data, split_ratio=split_ratio, y_col=target, shuffle=True
    )
    DataSet.data = DataSet.clean_data(DataSet.data)
    clf, report, best_parameters, best_index = cross_validation_grid_search(
        scores,
        RandomForestClassifier,
        DataSet.X_train,
        DataSet.X_test,
        DataSet.y_train,
        DataSet.y_test,
    )
    assert type(report) == type({})



from chembee_config.calibration.logistic_regression import LogisticRegressionClassifier
from chembee_config.calibration.naive_bayes import NaiveBayesClassifier
from chembee_config.calibration.svc import NaivelyCalibratedSVCRBF
from chembee_config.calibration.svc import NaivelyCalibratedSVCPolynomial
from chembee_config.calibration.random_forest import RandomForestClassifier
from chembee_config.calibration.knn import KNNClassifier
from chembee_config.calibration.mlp_classifier import NeuralNetworkClassifierRELU
from chembee_config.calibration.mlp_classifier import NeuralNetworkClassifierTanh


clf_list = [
    (LogisticRegressionClassifier, "Logistic Regression"),
    (NaiveBayesClassifier, "Naive Bayes"),
    (NaivelyCalibratedSVCRBF, "SVC RBF Kernel"),
    (NaivelyCalibratedSVCPolynomial, "SVC Polynomial Kernel degree 3"),
    (RandomForestClassifier, "Random forest"),
    (KNNClassifier, "k-nearest neighbor"),
    (NeuralNetworkClassifierRELU, "Multilayer Perceptron RELU"),
    (NeuralNetworkClassifierTanh, "Multilayer Perceptron Tanh"),
]



from chembee_datasets.DataSet import DataSet
import pandas as pd
from sklearn.model_selection import train_test_split
from rdkit import Chem
from rdkit.Chem import (
    Descriptors,
    Descriptors3D,
    rdMolDescriptors,
    PandasTools,
)

from file_utils import prepare_file_name_saving


class BioDegDataSet(DataSet):

    """
    The split ration is set to 0.7 for the feature extraction
    """

    name = "biodeg"

    def __init__(self, data_set_path, target, split_ratio=0.7):

        self.data, self.mols = self.load_data_set(data_set_path)
        self.split_ratio = split_ratio
        (
            self.X_train,
            self.X_test,
            self.y_train,
            self.y_test,
        ) = self.make_train_test_split(self.data, self.split_ratio, y_col=target)

        self.feature_names = self.get_feature_names(self.data, target=target)

    def clean_data(self, data):

        data = data.drop(columns=["SMILES", "Dataset", "CASRN", "ID"])
        data = data.convert_dtypes()
        bad_types = data.select_dtypes(
            exclude=["string", "int64", "float64"]
        ).columns.to_list()
        data = data.drop(columns=bad_types)
        return data

    def load_data_set(self, file_name: str):
        """
        The load_data function loads the data from a sdf file and returns it as a Pandas DataFrame.

        :param file_path:str: Used to Specify the location of the.
        :return: A dataframe with the following columns:.

        :doc-author: Trelent
        """
        mols = Chem.SDMolSupplier(file_name)
        frame = PandasTools.LoadSDF(
            file_name,
            smilesName="SMILES",
            molColName="Molecule",
            includeFingerprints=True,
            removeHs=False,
            strictParsing=True,
        )
        return frame, mols

    def load_data_set_from_csv(self, file_name: str) -> pd.DataFrame:

        return pd.from_csv(file_name)

    def make_train_test_split(self, data, split_ratio: float, y_col: str, shuffle=True):

        X = data.drop([y_col], axis=1)
        y = data[y_col]
        train_samples = int(split_ratio * len(X))
        X_train, X_test, y_train, y_test = train_test_split(
            X,
            y,
            shuffle=shuffle,
            test_size=len(X) - train_samples,
        )
        return X_train, X_test, y_train, y_test

    def save_data_csv(self, data: pd.DataFrame, file_name, prefix):

        assert type(data) == type(pd.DataFrame()), "Data must be a pandas.DataFrame"
        file_name = prepare_file_name_saving(
            prefix=prefix, file_name=file_name, ending=".csv"
        )
        data.to_csv(file_name)

    def save_data_sdf(self, data, file_name, prefix, molColName="Molecule"):

        file_name = prepare_file_name_saving(
            prefix=prefix, file_name=file_name, ending=".sdf"
        )
        PandasTools.WriteSDF(
            data, file_name, molColName="Molecule", properties=list(data.columns)
        )

    def get_feature_names(self, data, target):

        if not target:
            data = data.drop(columns=[target])
        return data.columns.to_list()



import sys, os
import numpy as np
import matplotlib
import matplotlib.pyplot as plt

from file_utils import prepare_file_name_saving

import logging

logging.basicConfig(
    format="%(levelname)s:%(asctime)s %(message)s",
    datefmt="%m/%d/%Y %I:%M:%S %p",
    level=logging.INFO,
    filename="chembee_plotting.log",
)


def plot_collection(metrics_json: dict, file_name: str, prefix: str):

    scalar_metrics = metrics_json["scalar"]
    array_metrics = metrics_json["array"]
    matrix_metrics = metrics_json["matrix"]

    plot_bar_chart_collection(scalar_metrics, file_name, prefix)
    plot_roc_chart_collection(array_metrics, file_name, prefix)
    plot_heat_map_collection(matrix_metrics, file_name, prefix)


def plot_heat_map_collection(metrics_json: dict, file_name: str, prefix: str):
    algs, metrics, metrics_storage = init_collection_plot(
        metrics_json, metric_type="scalar"
    )
    for metric in metrics:
        metrics_storage = []
        for i in range(len(algs)):
            metrics_storage.append(metrics_json[algs[i]][metric])


def plot_roc_chart_collection(metrics_json: dict, file_name: str, prefix: str):

    # TODO: Need to do this for multi_class, too
    # Not very elegant here the numbering is bad

    algs, metrics, metrics_storage = init_collection_plot(
        metrics_json, metric_type="scalar"
    )
    for i in range(len(algs)):
        plot_roc_chart(
            metrics_json[algs[i]]["fpr"],
            metrics_json[algs[i]]["tpr"],
            metrics_json[algs[i]]["roc_auc"],
            file_name="roc_auc_" + algs[i] + "_" + file_name,
            prefix=prefix,
        )


def plot_bar_chart_collection(metrics_json: dict, file_name: str, prefix: str):
    """
    The plot_bar_chart_collection function plots a collection of bar charts, one for each metric in the metrics_json.
    The metrics_json is expected to be a dictionary with keys corresponding to algorithm names and values being dictionaries
    of scalar values (one value per metric). The file name is expected to be the same for all plots.

    :param metrics_json:dict: Used to Store the values of all metrics for each algorithm.
    :param file_name:str: Used to Specify the name of the file that is generated by this function.
    :param prefix:str: Used to Distinguish between different types of plots.
    :return: The metrics_storage array.

    :doc-author: Trelent
    """

    algs, metrics, metrics_storage = init_collection_plot(
        metrics_json, metric_type="scalar"
    )
    metrics_store = np.zeros((len(metrics), len(algs)))
    for i in range(len(metrics)):
        metric = metrics[i]
        for j in range(len(algs)):
            try:
                metrics_storage[i, j] = metrics_json[algs[j]][metric]
            except:
                raise RuntimeError(
                    "Either value for alg:"
                    + str(algs[j])
                    + " or metric:"
                    + str(metric)
                    + " is not a valid value"
                )
        plot_bar_chart(
            algs,
            metrics_storage[i, :],
            y_label=metric,
            file_name=metric + "_" + file_name,
            prefix=prefix,
        )


def plot_roc_chart(
    fpr: list, tpr: list, roc_auc: list, prefix: str, file_name: str = "roc_auc_curve"
):
    """
    The plot_roc_chart function takes in a list of false positive rates, true positive rates, and the area under the curve.
    It then plots these values on a graph and saves it to file_name.png

    :param fpr:list: Used to Plot the false positive rate.
    :param tpr:list: Used to Plot the true positive rate.
    :param roc_auc:list: Used to Plot the area under the curve.
    :param prefix:str: Used to Add a prefix to the file name.
    :param file_name:str="roc_auc_curve": Used to Specify the name of the file to be saved.
    :return: The area under the curve (auc).

    :doc-author: Trelent
    """

    "Needs x,y data, check here for more information"
    matplotlib.rcParams.update({"font.size": 32})
    fig = plt.figure(figsize=(15, 15))
    lw = 4
    plt.plot(
        1 - fpr,  # compare test with standard function
        1 - tpr,
        lw=lw,
        label="ROC curve %s" % str(roc_auc),
    )
    plt.plot([0, 1], [0, 1], color="navy", lw=lw, linestyle="--")
    plt.xlim([0.0, 1.0])
    plt.ylim([0.0, 1.05])
    plt.xlabel("False Positive Rate")
    plt.ylabel("True Positive Rate")
    plt.legend(loc="lower right")
    fig.tight_layout()
    file_name = prepare_file_name_saving(
        prefix=prefix, file_name=file_name, ending=".png"
    )
    plt.savefig(file_name)
    plt.clf()
    plt.cla()
    plt.close()


def plot_heat_map():
    pass


def plot_grouped_bar_chart(
    labels: list,
    data_names: list,
    data: np.ndarray,
    file_name: str,
    prefix: str,
    y_label: str = "Score",
):
    """
    The plot_grouped_bar_chart function creates a grouped bar chart.
    The function takes the following parameters:
        - labels: list of strings, each string is a label for one group of bars;
        - data_names: list of strings, each string is a name for one bar in the plot;
        - data: numpy array with shape (len(labels), len(data_names)), contains values to be plotted as bars;
        - file_name (optional): str, name under which the plot will be saved. If not specified it will not save anything;
         prefix (optional): str, prefix that should be added to all files created by this function. If not specified no prefix will be added.

    :param labels:list: Used to Set the labels for each bar.
    :param data_names:list: Used to Label the bars.
    :param data:np.ndarray: Used to Pass the data to be plotted.
    :param file_name:str: Used to Specify the name of the file to be saved.
    :param prefix:str: Used to Identify the type of data that is being plotted.
    :param y_label:str="Score": Used to Set the y label of the plot.
    :return: A plot of the data.

    :doc-author: Trelent
    """

    # the problem here is that labels, and data entities are not the same. But viz is necessary to transport information. Anyhow, I need those ROC curves.

    file_name = prepare_plot_file_name(prefix, file_name)
    matplotlib.rcParams.update({"font.size": 22})
    fig = plt.figure(figsize=(15, 15))
    x = np.arange(len(data_names))  # the label locations
    width = 0.8 / len(data_names)  # the width of the bars

    fig, ax = plt.subplots()
    for i in range(len(labels)):
        rects = ax.bar(x - width / len(labels), data[i, :], width, label=labels[i])
        ax.bar_label(rects1, padding=len(labels) + 1)

    # Add some text for labels, title and custom x-axis tick labels, etc.
    ax.set_ylabel("Scores")
    ax.set_xticks(x, labels)
    ax.legend()
    fig.tight_layout()
    plt.savefig(file_name)
    plt.cla()
    plt.clf()
    plt.close()


def plot_bar_chart(
    algs: list, metrics: list, file_name: str, prefix: str, y_label: str, **kwargs
):
    """
    The plot_bar_chart function plots a bar chart of the given data.

    :param algs:list: Used to Specify the list of algorithms for which the metrics should be plotted.
    :param metrics:list: Used to Specify the metrics that should be plotted.
    :param file_name:str: Used to Specify the name of the file to save.
    :param prefix:str: Used to Specify the prefix for the file name.
    :return: A file name.

    :doc-author: Trelent
    """
    y_label = y_label
    matplotlib.rcParams.update({"font.size": 22})
    fig = plt.figure(figsize=(15, 15))
    plt.bar(algs, metrics)
    file_name = prepare_file_name_saving(
        prefix=prefix, file_name=file_name, ending=".png"
    )
    plt.ylabel(y_label)
    fig.tight_layout()
    fig.savefig(file_name)
    plt.clf()
    plt.cla()
    plt.close()
    logging.info("Plotted " + str(file_name))


def init_collection_plot(metrics_json, metric_type=""):
    """
    The init_collection_plot function initializes the plot for a collection of algorithms.
    It takes as input a metrics_json file and returns an array of values that will be plotted.

    :param metrics_json: Used to Store the metrics of each algorithm.
    :param metric_type="": Used to Specify the type of metric that we are plotting.
    :return: The metrics_storage variable.

    :doc-author: Trelent
    """

    algs, metrics = parse_metrics_output(metrics_json)
    metrics_storage = np.zeros((len(metrics), len(algs)))

    return algs, metrics, metrics_storage


def parse_metrics_output(metrics_collection: dict) -> dict:
    """
    The parse_metrics_output function takes a dictionary of metrics and returns a dictionary of dictionaries.
    The outermost key is the algorithm name, and the value is another dictionary with keys 'precision', 'recall',
    and 'fscore'. The values are lists containing precision, recall, and fscore for each class in order.

    :param metrics_collection:dict: Used to Store the metrics of each algorithm.
    :return: A dictionary of dictionaries.

    :doc-author: Trelent
    """

    algs = list(metrics_collection.keys())
    metrics = list(metrics_collection[algs[0]].keys())
    return algs, metrics


if __name__ == "__main__":
    sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

    from datasets.BreastCancer import BreastCancerDataset
    from actions.evaluation import screen_classifier_for_metrics
    from plotting.evaluation import plot_barchart_collection

    DataSet = BreastCancerDataset(split_ratio=0.8)
    metrics = screen_classifier_for_metrics(
        X_train=DataSet.X_train,
        X_test=DataSet.X_test,
        y_train=DataSet.y_train,
        y_test=DataSet.y_test,
    )
    plot_barchart_collection(
        metrics, file_name=DataSet.name + "_evaluation", prefix="plots/evaluation/"
    )



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from math import pi

from sklearn.inspection import DecisionBoundaryDisplay
from file_utils import prepare_file_name_saving

# set the standard parameters

plt.rcParams["font.size"] = "20"
plt.tight_layout()


def plot_comparison_4(
    models,
    titles,
    X,
    y,
    prefix=None,
    file_name="SVC_benchmark",
    feature_names=["Feature 1", "Feature 2"],
    response_method="predict",
):
    """
    The plot_comparision_4 function plots the decision boundaries for a set of models.
    It takes as input:
    - models: A list of scikit learn model objects.  Each model object must have a .predict method that takes in an array of features and returns predictions based on those features.  The plot_comparision_4 function uses these predictions to create the decision boundary for each model, and then plots them together in a single figure.
    - titles: A list with one element for each element in 'models'.  Each string is used to label the corresponding plot's legend, so make sure that they are unique!
    - X: An array containing feature values used to train/evaluate the models passed via 'models'.   These feature values should be scaled between 0 and 1 (if not already done).   The first two columns should correspond to x coordinates while the second two columns should correspond to y coordinates (i.e., this is an Nx4 matrix).  This argument corresponds directly with what you would pass into plt.scatter(...), so if you have already scaled your data, then just pass it here without re-scaling it!
    - y: An array containing labels corresponding with observations contained within X (i.e., this

    :param models: Used to Pass a list of models that should be compared.
    :param titles: Used to Give a name to each model in the plot.
    :param X: Used to Define the data that will be used for training and testing.
    :param y: Used to Define the target variable.
    :param prefix=None: Used to Define a string that will be added to the name of the file when it is saved.
    :param file_name="SVC_benchmark": Used to Save the plot as an image.
    :param feature_names=["Feature1": Used to Specify the labels for the x and y axis.
    :param "Feature2"]: Used to Specify which feature is used for the x-axis.
    :return: The subplot of the models.

    :doc-author: Trelent
    """

    fig, sub = plt.subplots(2, 2, figsize=(20, 13))
    plt.subplots_adjust(wspace=0.4, hspace=0.4)
    make_comparison_plot(
        fig=fig,
        sub=sub,
        models=models,
        titles=titles,
        X=X,
        y=y,
        file_name=file_name,
        feature_names=feature_names,
        prefix=prefix,
        response_method=response_method,
    )


def plot_comparison_3(
    models,
    titles,
    X,
    y,
    prefix=None,
    file_name="SVC_benchmark",
    feature_names=["Feature 1", "Feature 2"],
    response_method="predict",
):
    fig, sub = plt.subplots(1, 3, figsize=(20, 13))
    plt.subplots_adjust(wspace=0.4, hspace=0.4)
    make_comparison_plot(
        fig=fig,
        sub=sub,
        models=models,
        titles=titles,
        X=X,
        y=y,
        file_name=file_name,
        feature_names=feature_names,
        prefix=prefix,
        response_method=response_method,
    )


def plot_comparison_2():
    pass


def plot_comparison_1():
    pass


def make_comparison_plot(
    fig,
    sub,
    models,
    titles,
    X,
    y,
    prefix=None,
    file_name="SVC_benchmark",
    feature_names=["Feature 1", "Feature 2"],
    response_method="predict",
):
    """
    The make_comparison_plot function creates a plot comparing the decision boundaries of several models.
    It takes as input:
        - fig, a matplotlib figure object to be saved to file (figsize=(12,8))
        - sub, an array of axes objects on which plotting will occur (nrows=2, ncols=3)
        - models: an array of sklearn classifiers for which decision boundaries will be plotted.
            Note that these must have been fitted already with fit()!

        The function then creates plots comparing the decision boundary and prediction values for each model in turn.

        It returns nothing.

    :param fig: Used to Pass the figure object to which we want to plot.
    :param sub: Used to Create a subplot grid.
    :param models: Used to Pass the classifiers that will be used in the plot.
    :param titles: Used to Set the title of each subplot.
    :param X: Used to Pass the data to be plotted.
    :param y: Used to Specify the response variable.
    :param prefix=None: Used to Make sure that the file_name is saved in the current working directory.
    :param file_name="SVC_benchmark": Used to Create a file_name for the plot.
    :param feature_names=["Feature1": Used to Label the axes of the plot.
    :param "Feature2"]: Used to Specify the name of the x and y axis.
    :return: A plot of the decision boundaries for a set of classifiers.

    :doc-author: Trelent
    """

    X0, X1 = X[:, 0], X[:, 1]

    for clf, title, ax in zip(models, titles, sub.flatten()):
        disp = DecisionBoundaryDisplay.from_estimator(
            clf,
            X,
            response_method=response_method,
            cmap=plt.cm.coolwarm,
            alpha=0.7,
            ax=ax,
            xlabel=feature_names[0],
            ylabel=feature_names[1],
        )
        ax.scatter(X0, X1, c=y, cmap=plt.cm.coolwarm, s=20, edgecolors="k")
        ax.set_xticks(())
        ax.set_yticks(())
        ax.set_title(title)

    file_name = prepare_file_name_saving(
        prefix=prefix, file_name=file_name, ending=".png"
    )
    fig.savefig(file_name)


def plot_feature_importances():
    """ """


def save_fig(ax, name: str, save_path=None, dpi=300):
    """Frequently used snippet to save a plot

    Args:
        ax ([type]): Seaborn axis
        name (str): Name of the plot
        save_path ([type], optional): Where to save the plot. Defaults to None:str.
        dpi (int, optional): Resolution of the plot. Defaults to 300.

    Returns:
        (None)
    """
    if save_path is None:
        save_path = name
    else:
        save_path.joinpath(name)
    fig = ax.get_figure()
    fig.savefig(save_path, dpi=dpi)
    return None


plotting_map_comparison = {
    1: plot_comparison_1,
    2: plot_comparison_2,
    3: plot_comparison_3,
    4: plot_comparison_4,
}
if __name__ == "__main__":

    data = pd.read_csv("converted_data.csv")
    biodeg = data[data["ReadyBiodegradability"] == 1]
    n_biodeg = data[data["ReadyBiodegradability"] == 0]
    polar_plot(biodeg, name="BiodegPolar.png")
    polar_plot(n_biodeg, name="NBiodegPolar.png")
